// AnimationEngine.java

import java.applet.AudioClip;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferStrategy;

import java.util.ArrayList;

public class AnimationEngine
{
   private GraphicsDevice monitor;

   private int width;

   private int height;

   private int fps;

   private ArrayList<Animation> animations;

   private boolean done;

   private BufferStrategy strategy;

   private Audio audio;

   private boolean playAudio = true;

   private long startMillis;

   private boolean clipPlayed;

   public AnimationEngine (int width, int height, int bitDepth, int fps)
          throws NoFullScreenException, UnsupportedDisplayModeException
   {
      GraphicsEnvironment ge;
      ge = GraphicsEnvironment.getLocalGraphicsEnvironment ();

      monitor = ge.getDefaultScreenDevice ();
      if (!monitor.isFullScreenSupported ())
          throw new NoFullScreenException ();

      DisplayMode myMode;
      myMode = new DisplayMode (width, height, bitDepth,
                                DisplayMode.REFRESH_RATE_UNKNOWN);

      DisplayMode [] availModes = monitor.getDisplayModes ();

      int i = 0;
      for (i = 0; i < availModes.length; i++)
           if (availModes [i].getWidth () == myMode.getWidth () &&
               availModes [i].getHeight () == myMode.getHeight () &&
               availModes [i].getBitDepth () == myMode.getBitDepth ())
               break;

      if (i == availModes.length)
          throw new UnsupportedDisplayModeException ();

      this.width = width;
      this.height = height;
      this.fps = fps;

      animations = new ArrayList<Animation> ();

      Frame frame = new Frame ();
      frame.setUndecorated (true);
      frame.setResizable (false);
      frame.setIgnoreRepaint (true);

      frame.addKeyListener (new KeyAdapter ()
                            {
                                public void keyPressed (KeyEvent e)
                                {
                                   if (e.getKeyCode () == KeyEvent.VK_ESCAPE)
                                       done = true;
                                }

                                public void keyTyped (KeyEvent e)
                                {
                                   if (e.getKeyChar () == 'a')
                                       playAudio = !playAudio;
                                }
                            });

      frame.addMouseListener (new MouseAdapter ()
                              {
                                  public void mousePressed (MouseEvent e)
                                  {
                                     done = true;
                                  }
                              });

      monitor.setFullScreenWindow (frame);
      monitor.setDisplayMode (myMode);

      frame.createBufferStrategy (2);
      strategy = frame.getBufferStrategy ();

      Toolkit toolkit = Toolkit.getDefaultToolkit ();
      Image imWhite = ResourceLoader.loadImage ("images/misc/white.gif");
      Cursor hidden = toolkit.createCustomCursor (imWhite, new Point (0, 0),
                                                  "");
      frame.setCursor (hidden);
   }

   public void register (Animation animation)
   {
      animations.add (animation);
   }

   public void register (Audio audio)
   {
      this.audio = audio;
   }

   public void runAnimations ()
   {
      int frameDelay = 1000/fps;

      AudioClip ac = audio.getAudioClip ();

      int framesToWait = audio.getDelay ();

      int framesCounter = 0;

      while (!done)
      {
         startMillis = System.currentTimeMillis ();

         do
         {
            do
            {
                Graphics g = strategy.getDrawGraphics ();

                for (int i = 0; i < animations.size (); i++)
                     animations.get (i).render (g);

                g.dispose ();
            }
            while (strategy.contentsRestored ());

            strategy.show ();
         }
         while (strategy.contentsLost ());

         while (System.currentTimeMillis ()-startMillis < frameDelay)
         {
            if (!clipPlayed && ac != null && playAudio)
            {
                if (++framesCounter == framesToWait)
                {
                    ac.play ();
                    framesToWait = audio.getDelay ();
                    framesCounter = 0;
                }

                clipPlayed = true;
            }
         }

         clipPlayed = false;
      }
   }

   public void shutdown ()
   {
      monitor.setFullScreenWindow (null);
   }
}
