// UTS.java

import java.applet.AudioClip;

import java.awt.*;

public class UTS
{
   // screen width (in pixels)

   final static int WIDTH = 800;

   // screen height (in pixels)

   final static int HEIGHT = 600;

   // color mode -- 32 indicates true-color (32-bit) mode

   final static int BITDEPTH = 32;

   // animation frame rate, measured in frames per second

   final static int FPS = 20;

   public static void main (String [] args) throws Exception
   {
      // Create the animations.

      AngelFish af = new AngelFish (WIDTH, HEIGHT-AngelFish.height-
                                    rnd (Fish.VERTICAL_RANGE), WIDTH,
                                    HEIGHT, 3, AngelFish.LEFT, 0);

      Background bg = new Background ();

      Bubbles b1 = new Bubbles (168, 251-Bubbles.height, 3);
      Bubbles b2 = new Bubbles (376, 205-Bubbles.height, 3);
      Bubbles b3 = new Bubbles (679, 309-Bubbles.height, 3);


      Plant p1 = new Plant (20, HEIGHT-Plant.height, 3);
      Plant p2 = new Plant (WIDTH-Plant.width-20, HEIGHT-Plant.height, 3);

      TigerBarb tb1 = new TigerBarb (-TigerBarb.width,
                                     HEIGHT-TigerBarb.height-
                                     rnd (Fish.VERTICAL_RANGE), WIDTH,
                                     HEIGHT, 1, TigerBarb.RIGHT, 2);
      TigerBarb tb2 = new TigerBarb (-TigerBarb.width,
                                     HEIGHT-TigerBarb.height-
                                     rnd (Fish.VERTICAL_RANGE), WIDTH,
                                     HEIGHT, 2, TigerBarb.RIGHT, 1);

      ZebraFish zf1 = new ZebraFish (-ZebraFish.width,
                                     HEIGHT-ZebraFish.height-
                                     rnd (Fish.VERTICAL_RANGE), WIDTH,
                                     HEIGHT, 2, ZebraFish.RIGHT, 1);
      ZebraFish zf2 = new ZebraFish (-ZebraFish.width,
                                     HEIGHT-ZebraFish.height-
                                     rnd (Fish.VERTICAL_RANGE), WIDTH,
                                     HEIGHT, 3, ZebraFish.RIGHT, 0);

      // Create the audio.

      BubblesAudio ba = new BubblesAudio ();

      // Create an animation engine for a WIDTHxHEIGHTxBITDEPTH screen, where 
      // the overall animation rate does not exceed FPS. Enter fullscreen
      // exclusive mode and switch the display mode to WIDTHxHEIGHTxBITDEPTH.

      AnimationEngine engine;
      engine = new AnimationEngine (WIDTH, HEIGHT, BITDEPTH, FPS);

      // Register the animations with the animation engine. The order in
      // which animations are registered determines their Z-order -- which
      // animations cover other animations. Earlier registered animations are
      // covered by latter registered animations.

      engine.register (bg);
      engine.register (b1);
      engine.register (b2);
      engine.register (b3);
      engine.register (zf1);
      engine.register (af);
      engine.register (tb1);
      engine.register (tb2);
      engine.register (zf2);
      engine.register (p1);
      engine.register (p2);

      // Register the audio with the animation engine.

      engine.register (ba);

      // Run the animations and play the audio.

      engine.runAnimations ();

      // Shutdown the animation engine, reverting the display mode to the
      // display mode that was in effect before creating the animation engine.

      engine.shutdown ();

      // Terminate the application: System.exit is necessary.

      System.exit (0);
   }

   public static int rnd (int limit)
   {
      // Return a randomly-selected integer ranging from 0 through limit-1.

      return (int) (Math.random ()*limit);
   }
}

class AngelFish extends Fish
{
   private static Image [] imLFish;

   private static Image [] imRFish;

   static int height;

   static int width;

   static
   {
      imLFish = ResourceLoader.loadImages ("images/angelfish", "lfish", 8);
      height = imLFish [0].getHeight (null);
      width = imLFish [0].getWidth (null);
      imRFish = ResourceLoader.loadImages ("images/angelfish", "rfish", 8);
   }

   AngelFish (int x, int y, int dwidth, int dheight, int vx, int direction,
              int numFramesToSkip)
   {
      super (x, y, dwidth, dheight, vx, direction, numFramesToSkip, width,
             height, imLFish, imRFish);
   }
}

class Background implements Animation
{
   private Image imBackground;

   Background ()
   {
      imBackground = ResourceLoader.loadImage ("images/misc/background.gif");
   }

   public void render (Graphics g)
   {
      g.drawImage (imBackground, 0, 0, null);
   }
}

class Bubbles implements Animation
{
   private static Image imBubbles [];

   static int height;

   static
   {
      imBubbles = ResourceLoader.loadImages ("images/bubbles", "bubbles", 8);
      height = imBubbles [0].getHeight (null);
   }

   private int x;

   private int y;

   private int numFramesToSkip;

   private int nextFrame;

   private int frameCounter;

   Bubbles (int x, int y, int numFramesToSkip)
   {
      this.x = x;
      this.y = y;
      this.numFramesToSkip = numFramesToSkip;
   }

   public void render (Graphics g)
   {
      g.drawImage (imBubbles [nextFrame], x, y, null);

      if (numFramesToSkip == 0 || ++frameCounter == numFramesToSkip)
      {
          if (++nextFrame == imBubbles.length)
              nextFrame = 0;

          frameCounter = 0;
      }
   }
}

class BubblesAudio implements Audio
{
   private AudioClip ac;

   BubblesAudio ()
   {
      ac = ResourceLoader.loadAudioClip ("audios/bubbles.au");
   }

   public AudioClip getAudioClip ()
   {
      return ac;
   }

   public int getDelay ()
   {
      return UTS.FPS+UTS.rnd (100);
   }
}

class Fish implements Animation
{
   final static int LEFT = 0;

   final static int RIGHT = 1;

   final static int VERTICAL_RANGE = 400;

   private int x;

   private int y;

   private int dwidth;

   private int dheight;

   private int vx;

   private int direction;

   private int numFramesToSkip;

   private int width;

   private int height;

   private Image [] imLFish;

   private Image [] imRFish;

   private int nextFrame;

   private int frameCounter;

   Fish (int x, int y, int dwidth, int dheight, int vx, int direction,
         int numFramesToSkip, int width, int height, Image [] imLFish,
         Image [] imRFish)
   {
      this.x = x;
      this.y = y;
      this.dwidth = dwidth;
      this.dheight = dheight;
      this.vx = vx;
      this.direction = direction;
      this.numFramesToSkip = numFramesToSkip;
      this.width = width;
      this.height = height;
      this.imLFish = imLFish;
      this.imRFish = imRFish;
   }

   public void render (Graphics g)
   {
      g.drawImage ((direction == LEFT)
                   ? imLFish [nextFrame] : imRFish [nextFrame],
                   x, y, null);

      if (numFramesToSkip == 0 || ++frameCounter == numFramesToSkip)
      {
          if (direction == LEFT)
          {
              if (--nextFrame < 0)
                  nextFrame = imLFish.length-1;
          }
          else
          {
              if (++nextFrame == imRFish.length)
                  nextFrame = 0;
          }

          frameCounter = 0;
      }

      if (direction == LEFT)
      {
          x -= vx;
          if (x < -2*width)
          {
              switchDirection ();
              y = dheight-height-UTS.rnd (VERTICAL_RANGE);
          }
      }
      else
      {
          x += vx;
          if (x >= dwidth+width)
          {
              switchDirection ();
              y = dheight-height-UTS.rnd (VERTICAL_RANGE);
          }
      }
   }

   private void switchDirection ()
   {
      direction = (direction == LEFT) ? RIGHT : LEFT;
   }
}

class Plant implements Animation
{
   private static Image imPlant [];

   static int height;

   static int width;

   static
   {
      imPlant = ResourceLoader.loadImages ("images/plant", "plant", 3);
      height = imPlant [0].getHeight (null);
      width = imPlant [0].getWidth (null);
   }

   private int x;

   private int y;

   private int numFramesToSkip;

   private int nextFrame;

   private int frameCounter;

   Plant (int x, int y, int numFramesToSkip)
   {
      this.x = x;
      this.y = y;
      this.numFramesToSkip = numFramesToSkip;
   }

   public void render (Graphics g)
   {
      g.drawImage (imPlant [nextFrame], x, y, null);

      if (numFramesToSkip == 0 || ++frameCounter == numFramesToSkip)
      {
          if (++nextFrame == imPlant.length)
              nextFrame = 0;

          frameCounter = 0;
      }
   }
}

class TigerBarb extends Fish
{
   private static Image [] imLFish;

   private static Image [] imRFish;

   static int height;

   static int width;

   static
   {
      imLFish = ResourceLoader.loadImages ("images/tigerbarb", "lfish", 8);
      height = imLFish [0].getHeight (null);
      width = imLFish [0].getWidth (null);
      imRFish = ResourceLoader.loadImages ("images/tigerbarb", "rfish", 8);
   }

   TigerBarb (int x, int y, int dwidth, int dheight, int vx, int direction,
              int numFramesToSkip)
   {
      super (x, y, dwidth, dheight, vx, direction, numFramesToSkip, width,
             height, imLFish, imRFish);
   }
}

class ZebraFish extends Fish
{
   private static Image [] imLFish;

   private static Image [] imRFish;

   static int height;

   static int width;

   static
   {
      imLFish = ResourceLoader.loadImages ("images/zebrafish", "lfish", 8);
      height = imLFish [0].getHeight (null);
      width = imLFish [0].getWidth (null);
      imRFish = ResourceLoader.loadImages ("images/zebrafish", "rfish", 8);
   }

   ZebraFish (int x, int y, int dwidth, int dheight, int vx, int direction,
              int numFramesToSkip)
   {
      super (x, y, dwidth, dheight, vx, direction, numFramesToSkip, width,
             height, imLFish, imRFish);
   }
}
